'' =================================================================================================
''
''   File....... jm_74c922.spin
''   Purpose.... buffers input from 74C922
''   Author..... Jon "JonnyMac" McPhalen (aka Jon Williams)
''               Copyright (c) 2010-2011 Jon McPhalen
''               -- see below for terms of use
''   E-mail..... jon@jonmcphalen.com
''   Started.... 
''   Updated.... 10 JAN 2011
''
'' =================================================================================================


var

  long  cog

  long  kready                                                  ' key ready pin from MM74C922
  long  klsb                                                    ' LSB (A) pin of key bits outputs

  byte  buffer[16]                                              ' keystroke buffer
  byte  bufhead                                                 ' buffer pointers
  byte  buftail


pub start(rdy, lsb) | ok

'' Starts "background" 74C922 cog
'' -- rdy = key ready pin from 74C922
'' -- lsb = lsb (A) pin of 74C922 key bits outputs
''          * other pins are contiguous, in ascending order

  finalize                                                      ' stop if already running 

  longmove(@kready, @rdy, 2)                                    ' set pins
  flush                                                         ' clear buffer and pointers
  ok := cog := cognew(@entry, @kready) + 1                      ' start the cog

  return ok


pub finalize

'' Stops cog (if running)

  if cog
    cogstop(cog~ - 1)


pub getkey | key

'' Returns key if available, otherwise returns -1

  if (buftail <> bufhead)
    key := buffer[buftail]
    buftail := (buftail + 1) & $0F
    return key

  else
    return -1


pub waitkey | key

'' Returns key if available
'' -- will wait if buffer is empty

  repeat
    key := getkey
  until (key <> -1)

  return key


pub bufsize

'' Returns the number of key press events waiting in buffer (0 - 15)

  return ||(bufhead - buftail)


pub pressed

'' Returns state of key ready pin (true or false)
'' -- can be used to determine if last key is being held down

  return (ina[kready] == 1)


pub waitrelease

'' Holds program until last key is released

  if (cog)
    waitpne(|< kready, |< kready, 0)


pub flush

'' Flushes keyboard buffer and resets pointers

  bytefill(@buffer, 0, 18)
  

dat

                        org     0

entry                   mov     dira, #0                        ' all inputs

                        mov     tmp1, par                       ' start of structure
                        rdlong  tmp2, tmp1                      ' read ready pin
                        
                        mov     rdymask, #1                     ' create pin mask
                        shl     rdymask, tmp2

                        add     tmp1, #4
                        rdlong  lsbpin, tmp1                    ' read lsb pin of key group

                        add     tmp1, #4
                        mov     bufpntr, tmp1                   ' save address of buffer[0]

                        add     tmp1, #16                       ' skip over 16-byte buffer
                        mov     headpntr, tmp1                  ' save address of bufhead

keyin                   waitpeq rdymask, ina                    ' wait for ready signal
                        mov     tmp1, #5                        ' let outputs settle (Tpd)
:loop                   nop
                        djnz    tmp1, #:loop

                        mov     work, ina                       ' get key 
                        shr     work, lsbpin                    ' lsb align bits
                        and     work, KEY_MASK                  ' mask off unused bits

putbuf                  rdbyte  tmp1, headpntr                  ' tmp1 := bufhead
                        add     tmp1, bufpntr                   ' tmp1 := buffer[bufhead]
                        wrbyte  work, tmp1                      ' buffer[bufhead] := work
                        sub     tmp1, bufpntr                   ' tmp1 := bufhead 
                        add     tmp1, #1                        ' ++tmp1
                        and     tmp1, #$0F                      ' keep 0..15
                        wrbyte  tmp1, headpntr                  ' bufhead := tmp1

                        waitpne rdymask, ina                    ' force key release
                        mov     tmp1, #5                        ' let kready line drop
:loop                   nop                 
                        djnz    tmp1, #:loop

                        jmp     #keyin                          ' get next 

' --------------------------------------------------------------------------------------------------

KEY_MASK                long    %1111                           ' mask for key inputs

rdymask                 res     1                               ' pin mask for ready pin
lsbpin                  res     1                               ' lsb pin of key pins group

bufpntr                 res     1                               ' hub address of key buffer
headpntr                res     1                               ' hub address of head pointer
work                    res     1                               ' worskpace for key input

tmp1                    res     1                               ' work vars
tmp2                    res     1

                        fit     492                                    

                        
dat

{{

  Terms of Use: MIT License

  Permission is hereby granted, free of charge, to any person obtaining a copy of this
  software and associated documentation files (the "Software"), to deal in the Software
  without restriction, including without limitation the rights to use, copy, modify,
  merge, publish, distribute, sublicense, and/or sell copies of the Software, and to
  permit persons to whom the Software is furnished to do so, subject to the following
  conditions:

  The above copyright notice and this permission notice shall be included in all copies
  or substantial portions of the Software.

  THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR IMPLIED,
  INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY, FITNESS FOR A
  PARTICULAR PURPOSE AND NON-INFRINGEMENT. IN NO EVENT SHALL THE AUTHORS OR COPYRIGHT
  HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER LIABILITY, WHETHER IN AN ACTION OF
  CONTRACT, TORT OR OTHERWISE, ARISING FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE
  OR THE USE OR OTHER DEALINGS IN THE SOFTWARE.

}}  